from __future__ import annotations

import jijbench as jb
import numpy as np
import pandas as pd
import pytest


def test_record():
    inputs = [
        jb.ID(name="id1"),
        jb.Date(name="date1"),
        jb.Array(np.arange(5), name="array1"),
    ]
    data = pd.Series(inputs)
    jb.Record(data)


def test_artifact():
    from jijbench.typing import ArtifactDataType

    data: ArtifactDataType = {
        "0": {
            "id": jb.ID(name="id1"),
            "date": jb.Date(name="date1"),
            "array": jb.Array(np.arange(5), name="array1"),
        },
        "1": {
            "id": jb.ID(name="id2"),
            "date": jb.Date(name="date2"),
            "array": jb.Array(np.arange(5), name="array2"),
        },
    }
    jb.Artifact(data)


def test_table():
    inputs = [
        [
            jb.ID(name="id1"),
            jb.Date(name="date1"),
            jb.Array(np.arange(5), name="array1"),
        ]
    ]
    data = pd.DataFrame(inputs)
    jb.Table(data)


def test_record_invalid_data():
    with pytest.raises(TypeError):
        jb.Record(1)

    with pytest.raises(TypeError):
        inputs = [
            "id",
            pd.Timestamp.now(),
            np.arange(5),
        ]
        data = pd.Series(inputs)
        jb.Record(data)


def test_artifact_invalid_data():
    with pytest.raises(TypeError):
        jb.Artifact(1)

    with pytest.raises(TypeError):
        data = {
            "0": {
                "id": "id",
                "date": pd.Timestamp.now(),
                "array": np.arange(5),
            }
        }
        jb.Artifact(data)


def test_table_invalid_data():
    with pytest.raises(TypeError):
        jb.Table(1)

    with pytest.raises(TypeError):
        inputs = [
            [
                "id1",
                pd.Timestamp.now(),
                np.arange(5),
            ]
        ]
        data = pd.DataFrame(inputs)
        jb.Table(data)


def test_record_append():
    factory = jb.functions.RecordFactory()

    inputs1 = [
        jb.ID(name="id1"),
        jb.Date(name="date1"),
        jb.Array(np.arange(5), name="array1"),
    ]
    inputs2 = [
        jb.ID(name="id2"),
        jb.Date(name="date2"),
        jb.Array(np.arange(5), name="array2"),
    ]

    name = "r1"
    r1 = factory(inputs1, name=name)
    r2 = factory(inputs2, name="r2")

    r1.append(r2)

    assert isinstance(r1, jb.Record)
    assert r1.name == name
    assert "id2" in r1.data.index
    assert "date2" in r1.data.index
    assert "array2" in r1.data.index
    assert r1.operator is not None
    assert len(r1.operator.inputs) == 2
    for i in ["id2", "date2", "array2"]:
        assert r1.data[i] == r2.data[i]


def test_table_append():
    factory = jb.functions.TableFactory()

    name = "t1"
    data = [jb.ID("id"), jb.Date(), jb.Array(np.arange(5), "array")]
    r1 = jb.Record(pd.Series(data), "a")
    table = factory([r1], name=name)

    r2 = jb.Record(pd.Series(data), "b")
    table.append(r2)

    assert isinstance(table, jb.Table)
    assert table.name == name
    assert table.data.index[0] == "a"
    assert table.data.index[1] == "b"
    assert table.operator is not None
    assert len(table.operator.inputs) == 2
    for i, d in enumerate(table.data.loc["a"]):
        assert d == table.operator.inputs[0].data.loc["a", i]


def test_artifact_append():
    factory = jb.functions.ArtifactFactory()

    name = "a1"
    data = [jb.ID("id"), jb.Date(), jb.Array(np.arange(5), "array")]
    r1 = jb.Record(pd.Series(data), "a")
    artifact = factory([r1], name=name)

    r2 = jb.Record(pd.Series(data), "b")
    artifact.append(r2)

    assert isinstance(artifact, jb.Artifact)
    assert artifact.name == name
    assert "a" in artifact.data
    assert "b" in artifact.data
    assert artifact.operator is not None
    assert len(artifact.operator.inputs) == 2
    for i, d in artifact.data["a"].items():
        assert d == artifact.operator.inputs[0].data["a"][i]


def test_experiment_append():
    ename = "test"
    e = jb.Experiment(name=ename)

    for i in range(3):
        with e:
            data = [
                jb.Date(),
                jb.Number(i, "num"),
                jb.Array(np.arange(5), "array"),
            ]
            record = jb.functions.RecordFactory()(data)
            e.append(record)

    assert isinstance(e, jb.Experiment)
    assert e.operator is not None
    assert e.data[0].operator is not None
    assert e.data[1].operator is not None
    assert len(e.operator.inputs) == 2

    for i, index in zip(range(3), e.artifact):
        assert e.artifact[index]["num"] == i
        assert e.table.loc[index, "num"] == i


def test_ref():
    factory = jb.functions.RecordFactory()
    inputs1 = [
        jb.ID(name="id1"),
        jb.Date(name="date1"),
        jb.Array(np.arange(5), name="array1"),
    ]

    r1 = factory(inputs1)

    factory = jb.functions.TableFactory()
    table = r1.apply(factory)



from __future__ import annotations

import abc
import numpy as np
import pandas as pd
import jijmodeling as jm
import typing as tp
import warnings

from dataclasses import dataclass, field
from jijbench.functions.concat import Concat
from jijbench.functions.factory import ArtifactFactory, TableFactory
from jijbench.node.base import DataNode
from jijbench.typing import T, ArtifactDataType


@dataclass
class Container(DataNode[T]):
    """An abstract class for all Container classes that implements the methods to be
    followed by all child classes.
    """

    @abc.abstractmethod
    def __len__(self) -> int:
        """
        Perform the operation __len__.
        """
        pass

    @abc.abstractmethod
    def append(self, record: Record) -> None:
        """Append method to be implemented in the child classes.

        Args:
            record: the record to be appended.

        Returns:
            A data type of class T.
        """
        pass

    @abc.abstractmethod
    def view(self) -> T:
        """View method to be implemented in the child classes."""
        pass


@dataclass
class Record(Container[pd.Series]):
    """Data structure that maps data onto a `pandas.Series`.

    Attributes:
        data (pandas.Series): pandas series object.
        name (Hashable): The name of the record. Defaults to None.
    """

    data: pd.Series = field(default_factory=lambda: pd.Series(dtype="object"))
    name: tp.Hashable = None

    def __len__(self) -> int:
        """
        Perform the operation __len__.
        """
        return len(self.data)

    @classmethod
    def validate_data(cls, data: pd.Series) -> pd.Series:
        """
        Validate the data to ensure that it is a pandas Series and all elements of the Series are instances of DataNode.

        Args:
            data (pandas.Series): The data to be validated.

        Raises:
            TypeError: If the input data is not a pandas Series or if not all elements of the Series are instances of DataNode.

        Returns:
            The validated data.

        """
        data = cls._validate_dtype(data, (pd.Series,))
        if data.empty:
            return data
        else:
            if data.apply(lambda x: isinstance(x, DataNode)).all():
                return data
            else:
                raise TypeError(
                    f"All elements of {data.__class__.__name__} must be type DataNode."
                )

    @property
    def index(self) -> pd.Index:
        """Return the index of the Record data."""
        return self.data.index

    @index.setter
    def index(self, index: pd.Index) -> None:
        """Set the index of the Record data.

        Args:
            index (pd.Index): The index to set for the Record data.
        """
        self.data.index = index

    def append(self, record: Record) -> None:
        """Apeend a new Record to the current Record.

        Args:
            record (Record): The Record to be added.
        """
        concat: Concat[Record] = Concat()
        node = self.apply(concat, [record], name=self.name)
        self._init_attrs(node)

    def view(self) -> pd.Series:
        """Return the data of each DataNode in the Series as a new Series."""
        return self.data.apply(lambda x: x.data)


@dataclass
class Artifact(Container[ArtifactDataType]):
    """Data structure that maps data onto a `dict`.

    Attributes:
        data (ArtifactDataType): The data stored in the Artifact.
        name (Hashable): The name of the Artifact. Defaults to None.
    """

    data: ArtifactDataType = field(default_factory=dict)
    name: tp.Hashable = None

    def __len__(self) -> int:
        """
        Perform the operation __len__.
        """
        return len(self.data)

    @classmethod
    def validate_data(cls, data: ArtifactDataType) -> ArtifactDataType:
        """Validate the data stored in the Artifact.

        The data in the Artifact must be of type `dict`. The values stored in
        the `dict` must be of type `DataNode`.

        Args:
            data (ArtifactDataType): The data to be validated.

        Raises:
            TypeError: If the data is not of the correct type.

        Returns:
            ArtifactDataType: The validated data.
        """
        if data:
            data = cls._validate_dtype(data, (dict,))
            values = []
            for v in data.values():
                if isinstance(v, dict):
                    values += list(v.values())
                else:
                    raise TypeError(
                        f"Type of attibute data is {ArtifactDataType}. Input data is invaid."
                    )
            if all(map(lambda x: isinstance(x, DataNode), values)):
                return data
            else:
                raise TypeError(
                    f"Type of attibute data is {ArtifactDataType}. Input data is invaid."
                )
        else:
            return data

    def keys(self) -> tuple[tp.Hashable, ...]:
        """Return a tuple of keys."""
        return tuple(self.data.keys())

    def values(self) -> tuple[dict[tp.Hashable, DataNode], ...]:
        """Return a tuple of values."""
        return tuple(self.data.values())

    def items(
        self,
    ) -> tuple[tuple[tp.Hashable, dict[tp.Hashable, DataNode]], ...]:
        """Return a tuple of key-value pairs."""
        return tuple(self.data.items())

    def append(self, record: Record) -> None:
        """Append a new record to the Artifact.

        Args:
            record (Record): The data to be appended.
        """
        concat: Concat[Artifact] = Concat()
        other = ArtifactFactory()([record])
        node = self.apply(concat, [other], name=self.name)
        self._init_attrs(node)

    def view(self) -> ArtifactDataType:
        """Return the data of each DataNode in the dict as a new dict."""
        return {
            k: {name: node.data for name, node in v.items()}
            for k, v in self.data.items()
        }


@dataclass
class Table(Container[pd.DataFrame]):
    """Data structure that maps data onto a `pandas.DataFrame`.

    This class is one of Container. The element in the each cell is a DataNode.
    Table class extends the basic functionality of a `pandas.DataFrame` with the ability to store and manipulate `DataNode` objects.

    Attributes:
        data (pd.DataFrame): The actual data stored in the table.
        name (tp.Hashable): The name of the Table, which is used as an identifier.
    """

    data: pd.DataFrame = field(default_factory=pd.DataFrame)
    name: tp.Hashable = None

    def __len__(self) -> int:
        """
        Perform the operation __len__.
        """
        return len(self.data)

    @classmethod
    def validate_data(cls, data: pd.DataFrame) -> pd.DataFrame:
        """Validate the data to ensure it is a pandas DataFrame with all elements being of type `DataNode`.

        Args:
            data (pandas.DataFrame): The data to validate.

        Raises:
            TypeError: If the input data is not a pandas DataFrame or if it contains elements that are not of type `DataNode`.

        Returns:
            pandas.DataFrame: The validated data.
        """
        data = cls._validate_dtype(data, (pd.DataFrame,))
        if data.empty:
            return data
        else:
            if data.applymap(lambda x: isinstance(x, DataNode)).values.all():
                return data
            else:
                raise TypeError(
                    f"All elements of {data.__class__.__name__} must be type DataNode."
                )

    @property
    def index(self) -> pd.Index:
        """Return the index of the data in the Table object."""
        return self.data.index

    @index.setter
    def index(self, index: pd.Index) -> None:
        """Set the index of the data in the Table object.

        Args:
            index (pandas.Index): The new index for the data stored in the Table object.
        """
        self.data.index = index

    @property
    def columns(self) -> pd.Index:
        """Return the columns of the data in the Table object."""
        return self.data.columns

    @columns.setter
    def columns(self, columns: pd.Index) -> None:
        """Set the columns of the data in the Table object.

        Args:
            columns (pandas.Index): The new columns for the data stored in the Table object.
        """
        self.data.columns = columns

    def append(self, record: Record) -> None:
        """Append a new record to the Table.

        Args:
            record (Record): The data to be appended.
        """
        concat: Concat[Table] = Concat()
        other = TableFactory()([record])
        node = self.apply(concat, [other], name=self.name, axis=0)
        self._init_attrs(node)

    def view(self) -> pd.DataFrame:
        """Return the data of each DataNode in the pandas.DataFrame as a new pandas.DataFrame."""
        if self.data.empty:
            return self.data
        else:
            data = self.data.applymap(lambda x: x.data)
            sampleset_columns = [
                c for c in data.columns if isinstance(data[c][0], jm.SampleSet)
            ]
            if sampleset_columns:
                extracted = pd.concat(
                    [data[c].apply(self._extract) for c in sampleset_columns], axis=1
                )
                data.drop(sampleset_columns, axis=1, inplace=True)
                return pd.concat([data, extracted], axis=1)
            else:
                return data

    def _extract(self, sampleset: jm.SampleSet) -> pd.Series:
        """Extract data from jijmodeling.SampleSet object.

        This method extracts relevant data from a `jijmodeling.SampleSet`, such as the number of occurrences,
        energy, objective, constraint violations, number of samples, number of feasible samples, and the
        execution time. The extracted data is returned as a list of DataNode objects.


        Args:
            node (jijmodeling.SampleSet): A jijmodeling SampleSet from which to extract data.

        Returns:
            pd.Series: Extracted data from the SampleSet..
        """

        data = {}
        data["num_occurrences"] = np.array(sampleset.record.num_occurrences)
        data["energy"] = np.array(sampleset.evaluation.energy)
        data["objective"] = np.array(sampleset.evaluation.objective)

        constraint_violations = sampleset.evaluation.constraint_violations
        if constraint_violations:
            for k, v in constraint_violations.items():
                data[f"{k}_violations"] = np.array(v)

        data["num_samples"] = sum(sampleset.record.num_occurrences)
        data["num_feasible"] = sum(sampleset.feasible().record.num_occurrences)

        # TODO スキーマが変わったら修正
        solving_time = sampleset.measuring_time.solve
        if solving_time is None:
            execution_time = np.nan
            warnings.warn(
                "'solve' of jijmodeling.SampleSet is None. Give it if you want to evaluate automatically."
            )
        else:
            if solving_time.solve is None:
                execution_time = np.nan
                warnings.warn(
                    "'solve' of jijmodeling.SampleSet is None. Give it if you want to evaluate automatically."
                )
            else:
                execution_time = solving_time.solve
        data["execution_time"] = execution_time
        return pd.Series(data)



import os

from setuptools import Extension, find_namespace_packages, setup

setup(
    packages=find_namespace_packages(include=["jijbench*"]),
    ext_modules=[
        Extension(
            "jijbench.__marker__",
            [os.path.join("jijbench", "__marker__.c")],
        )
    ],
    package_data={
        "": ["*.json", "*.JSON"],
    },
    include_package_data=True,
)



from __future__ import annotations

import jijmodeling as jm
import typing as tp
import inspect
import itertools
import pathlib


from jijbench.consts.path import DEFAULT_RESULT_DIR
from jijbench.node.base import FunctionNode
from jijbench.elements.base import Callable
from jijbench.elements.date import Date
from jijbench.elements.id import ID
from jijbench.experiment.experiment import Experiment
from jijbench.functions.concat import Concat
from jijbench.functions.factory import RecordFactory
from jijbench.solver.base import Parameter, Solver
from jijbench.solver.jijzept import InstanceData, UserDefinedModel

if tp.TYPE_CHECKING:
    from jijzept.sampler.base_sampler import JijZeptBaseSampler


class Benchmark(FunctionNode[Experiment, Experiment]):
    """ "A class representing a benchmark.

    This class allows to define a benchmark as a collection of experiments
    over a set of parameters and solvers. The benchmark will be run sequentially
    or concurrently and the results of each experiment will be concatenated and
    returned as a single experiment.

    Attributes:
        params (dict[str, Iterable[Any]]): List of lists of parameters for the benchmark.
        solver (Callable | list[Callable]): List of solvers to be used in the benchmark.
        name (str | None, optional): Name of the benchmark.
    """

    def __init__(
        self,
        params: dict[str, tp.Iterable[tp.Any]],
        solver: tp.Callable | list[tp.Callable],
        name: str | None = None,
    ) -> None:
        """Initializes the benchmark with the given parameters and solvers.

        Args:
            params (dict[str, Iterable[Any]]): Dictionary of parameters for the benchmark.
                The keys should be the names of the parameters and the values should
                be iterables of the respective parameter values.
            solver (Callable | list[Callable]): A single solver or a list of solvers to be used in the benchmark.
                The solvers should be callable objects taking in a list of parameters.
            name (str | None, optional): Name of the benchmark. Defaults to None.

        Raises:
            TypeError: If the name is not a string.
        """
        if name is None:
            name = ID().data
        super().__init__(name)

        self.params = [
            [
                v if isinstance(v, Parameter) else Parameter(v, k)
                for k, v in zip(params.keys(), r)
            ]
            for r in itertools.product(*params.values())
        ]

        if isinstance(solver, tp.Callable):
            self.solver = [Solver(solver)]
        else:
            self.solver = [Solver(f) for f in solver]

    def __call__(
        self,
        inputs: list[Experiment] | None = None,
        concurrent: bool = False,
        autosave: bool = True,
        savedir: str | pathlib.Path = DEFAULT_RESULT_DIR,
    ) -> Experiment:
        """Executes the benchmark with the given parameters and solvers.

        Args:
            inputs (list[Experiment] | None, optional): A list of input experiments to be used by the benchmark. Defaults to None.
            concurrent (bool, optional): Whether to run the experiments concurrently or not. Defaults to False.
            autosave (bool, optional): Whether to automatically save the Experiment object after each run. Defaults to True.
            savedir (str | pathlib.Path, optional): The directory to save the Experiment object. Defaults to DEFAULT_RESULT_DIR.

        Returns:
            Experiment: The result of the benchmark as an Experiment object.
        """
        savedir = (
            savedir if isinstance(savedir, pathlib.Path) else pathlib.Path(savedir)
        )
        savedir /= self.name
        if inputs is None:
            inputs = [Experiment(autosave=autosave, savedir=savedir)]

        return super().__call__(
            inputs,
            concurrent=concurrent,
            autosave=autosave,
            savedir=savedir,
        )

    @property
    def name(self) -> str:
        """The name of the benchmark."""
        return str(self._name)

    @name.setter
    def name(self, name: str) -> None:
        """Sets the name of the benchmark.

        Args:
            name (str): The name to be set.

        Raises:
            TypeError: If the name is not a string.
        """
        if not isinstance(name, str):
            raise TypeError("Becnhmark name must be string.")
        self._name = name

    def operate(
        self,
        inputs: list[Experiment],
        concurrent: bool = False,
        autosave: bool = True,
        savedir: str | pathlib.Path = DEFAULT_RESULT_DIR,
    ) -> Experiment:
        """Performs the operations specified in the benchmark on the input experiments and returns the Experiment object.

        Args:
            inputs (list[Experiment]): A list of input experiments.
            concurrent (bool, optional): Whether to run the operations concurrently or not. Defaults to False.
            autosave (bool, optional): Whether to automatically save the Experiment object after each run. Defaults to True.
            savedir (str | pathlib.Path, optional): The directory to save the Experiment object. Defaults to DEFAULT_RESULT_DIR.

        Returns:
            Experiment: An Experiment object representing the results of the operations performed by the benchmark.
        """
        concat: Concat[Experiment] = Concat()
        name = inputs[0].name
        experiment = concat(inputs, name=name, autosave=autosave, savedir=savedir)
        if concurrent:
            return self._co()
        else:
            return self._seq(experiment)

    def _co(self) -> Experiment:
        raise NotImplementedError

    def _seq(
        self,
        experiment: Experiment,
    ) -> Experiment:
        for f in self.solver:
            for params in self.params:
                with experiment:
                    info = RecordFactory()(
                        [Date(), *params, Callable(f.function, str(f.name))]
                    )
                    ret = f(params)
                    record = Concat()([info, ret])

                    state = getattr(experiment, "state")
                    state.name = (self.name, experiment.name)
                    experiment.append(record)
                    experiment.data[1].index.names = [
                        "benchmark_id",
                        "experiment_id",
                        "run_id",
                    ]
        return experiment


def construct_benchmark_for(
    sampler: JijZeptBaseSampler,
    models: list[tuple[jm.Problem, jm.PH_VALUES_INTERFACE]],
    params: dict[str, tp.Iterable],
    name: str | None = None,
) -> Benchmark:
    """Create a Benchmark object.

    Args:
        sampler (JijZeptBaseSampler): The sampler to use for creating the benchmark.
        models (list[tuple[jijmodeling.Problem, jijmodeling.PH_VALUES_INTERFACE]]): A list of tuples containing jijmodeling.Problem and jijmodeling.PH_VALUES_INTERFACE.
        params (dict[str, Iterable]): The parameters to use for creating the benchmark.
        name (str | None, optional): The name of the benchmark. Defaults to None.

    Raises:
        KeyError: If the argument corresponding to jijmodeling.Prolblem is missing in sample_model.
        KeyError: If the argument corresponding to instance data is missing in sample_model.

    Returns:
        Benchmark: The constructed benchmark.
    """
    sample_model = getattr(sampler, "sample_model")
    sampler_parameters = inspect.signature(sample_model).parameters
    if "problem" in sampler_parameters:
        argname_problem = "problem"
    elif "model" in sampler_parameters:
        argname_problem = "model"
    else:
        raise KeyError(
            f"The argument corresponding to jijmodeling.Prolblem is missing in sample_model of {sampler.__class__.__name__}."
        )

    if "instance_data" in sampler_parameters:
        argname_instance_data = "instance_data"
    elif "feed_dict" in sampler_parameters:
        argname_instance_data = "feed_dict"
    else:
        raise KeyError(
            f"The argument corresponding to instance data is missing in sample_model of {sampler.__class__.__name__}."
        )

    bench = Benchmark(params, sample_model, name)

    additional_params = []
    for problem, instance_data in models:
        data = (problem, instance_data)
        data = UserDefinedModel.validate_data(data)
        additional_params.append(
            [
                Parameter(data[0], argname_problem),
                InstanceData(data[1], argname_instance_data),
            ]
        )
    bench.params = [p + ap for p in bench.params for ap in additional_params]

    return bench



from __future__ import annotations

import uuid

from dataclasses import dataclass, field
from jijbench.node.base import DataNode


@dataclass
class ID(DataNode[str]):
    """A class for ID.

    Attributes:
        data (str): The ID data. This is generated as a unique identifier if not specified at the time of instantiation.
        name (str): The name of the node.
    """

    data: str = field(default_factory=lambda: str(uuid.uuid4()))
    name: str = "id"

    @classmethod
    def validate_data(cls, data: str) -> str:
        """
        Validate the data for the ID.

        Args:
            data: The data to be stored in the instance.

        Raises:
            TypeError: If the data is not of the expected type.

        Returns:
            The validated data.
        """
        return cls._validate_dtype(data, (str,))



from __future__ import annotations

import inspect

from typing import Callable, Optional

import jijmodeling as jm
import jijzept as jz

from jijbench.exceptions import SolverFailedError


class CallableSolver:
    def __init__(self, solver):
        self._is_jijzept_sampler = False
        self.function = self._parse_solver(solver)
        self._name = self.function.__name__
        self._ret_names = (
            ["response", "decoded"] if solver in dir(DefaultSolver) else None
        )

    def __call__(self, **kwargs):
        parameters = inspect.signature(self.function).parameters
        is_kwargs = any([p.kind == 4 for p in parameters.values()])
        kwargs = (
            kwargs
            if is_kwargs
            else {k: v for k, v in kwargs.items() if k in parameters}
        )
        try:
            ret = self.function(**kwargs)
        except Exception as e:
            msg = f'An error occurred inside your solver. Please check implementation of "{self.name}". -> {e}'
            raise SolverFailedError(msg)
        return ret

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @property
    def ret_names(self):
        return self._ret_names

    @property
    def is_jijzept_sampler(self):
        return self._is_jijzept_sampler

    @ret_names.setter
    def ret_names(self, names):
        self._ret_names = names

    def to_named_ret(self, ret):
        if isinstance(ret, tuple):
            names = (
                self._ret_names
                if self._ret_names
                else [f"solver_return_values[{i}]" for i in range(len(ret))]
            )
            ret = dict(zip(names, ret))
        else:
            names = self.ret_names if self._ret_names else ["solver_return_values[0]"]
            ret = dict(zip(names, [ret]))
        return ret

    def _parse_solver(self, solver):
        if isinstance(solver, str):
            self._is_jijzept_sampler = True
            return getattr(DefaultSolver(), solver)
        elif isinstance(solver, Callable):
            return solver
        else:
            raise TypeError("solver of this type is not supported.")


class DefaultSolver:
    jijzept_config: Optional[str] = None
    dwave_config: Optional[str] = None
    jijzept_sampler_names = ["JijSASampler", "JijSQASampler", "JijSwapMovingSampler"]

    @property
    def JijSASampler(self):
        return self.jijzept_sa_sampler_sample_model

    @property
    def JijSQASampler(self):
        return self.jijzept_sqa_sampler_sample_model

    @property
    def JijSwapMovingSampler(self):
        return self.jijzept_swapmoving_sampler_sample_model

    @classmethod
    def jijzept_sa_sampler_sample_model(cls, problem, instance_data, **kwargs):
        return cls._sample_by_jijzept(
            jz.JijSASampler,
            problem,
            instance_data,
            **kwargs,
        )

    @classmethod
    def jijzept_sqa_sampler_sample_model(cls, problem, instance_data, **kwargs):
        return cls._sample_by_jijzept(
            jz.JijSQASampler, problem, instance_data, **kwargs
        )

    @classmethod
    def jijzept_swapmoving_sampler_sample_model(cls, problem, instance_data, **kwargs):
        return cls._sample_by_jijzept(
            jz.JijSwapMovingSampler, problem, instance_data, **kwargs
        )

    @staticmethod
    def _sample_by_jijzept(sampler, problem, instance_data, sync=True, **kwargs):
        sampler = sampler(config=DefaultSolver.jijzept_config)
        if sync:
            parameters = inspect.signature(sampler.sample_model).parameters
            kwargs = {k: w for k, w in kwargs.items() if k in parameters}
            response: jm.SampleSet = sampler.sample_model(
                problem, instance_data, sync=sync, **kwargs
            )
        else:
            response: jm.SampleSet = jz.response.JijModelingResponse.empty_response(
                jz.response.APIStatus.PENDING, sampler.client, kwargs["solution_id"]
            )
            response.get_result()

        return response



from __future__ import annotations

import jijbench as jb

from jijbench.datasets import TSP, TSPTW, Knapsack


def test_tsptw():
    target = TSPTW()

    assert len(target.small_instance()) > 0
    assert len(target.medium_instance()) > 0

    instance_name = target.instance_names("small")[0]
    ins_data = target.get_instance("small", instance_name)
    assert isinstance(ins_data, dict)


def test_tsptw_time_window_constraint():
    problem = jb.get_problem("TSPTW")
    print(str(problem.constraints["time-window-constraint"]))
    assert "1*20*1 - 1*x[i,j]" in str(problem.constraints["time-window-constraint"])


def test_tsp():
    target = TSP()

    assert len(target.small_instance()) > 0
    assert len(target.medium_instance()) > 0

    instance_name = target.instance_names("small")[0]
    ins_data = target.get_instance("small", instance_name)
    assert isinstance(ins_data, dict)


def test_knapsack():
    target = Knapsack()

    assert len(target.small_instance()) > 0
    assert len(target.medium_instance()) > 0

    instance_name = target.instance_names("small")[0]
    ins_data = target.get_instance("small", instance_name)
    assert isinstance(ins_data, dict)


def test_get_default_problem():
    problem = jb.get_problem("TSP")
    assert problem.name == "travelling-salesman"

    problem = jb.get_problem("TSPTW")
    assert problem.name == "travelling-salesman-with-time-windows"

    problem = jb.get_problem("Knapsack")
    assert problem.name == "knapsack"

    problem = jb.get_problem("BinPacking")
    assert problem.name == "bin-packing"

    problem = jb.get_problem("NurseScheduling")
    assert problem.name == "nurse-scheduling"


def test_get_default_instance_data():
    instance_data = jb.get_instance_data("TSP")

    assert isinstance(instance_data[0], tuple)
    assert isinstance(instance_data[0][0], str)
    assert isinstance(instance_data[0][1], dict)

    instance_data = jb.get_instance_data("NurseScheduling")
    assert isinstance(instance_data[0], tuple)
    assert isinstance(instance_data[0][0], str)
    assert isinstance(instance_data[0][1], dict)



from __future__ import annotations

import os, shutil

import jijmodeling as jm
import pytest

import jijbench as jb


@pytest.fixture(scope="function", autouse=True)
def pre_post_process():
    # preprocess
    yield
    # postprocess
    norm_path = os.path.normcase("./.jb_results")
    if os.path.exists(norm_path):
        shutil.rmtree(norm_path)


def test_success_probability(jm_sampleset: jm.SampleSet):
    from jijbench.functions.metrics import SuccessProbability

    sampleset = jb.SampleSet(jm_sampleset, "test_sampleset")

    opt_value = 3.0
    f = SuccessProbability()
    ps = f([sampleset], opt_value=opt_value)

    assert ps.data == 0.4


def test_feasible_rate(jm_sampleset: jm.SampleSet):
    from jijbench.functions.metrics import FeasibleRate

    sampleset = jb.SampleSet(jm_sampleset, "test_sampleset")

    f = FeasibleRate()
    feas_rate = f([sampleset])

    assert feas_rate.data == 0.7


def test_time_to_solution(jm_sampleset: jm.SampleSet):
    from jijbench.functions.metrics import TimeToSolution

    sampleset = jb.SampleSet(jm_sampleset, "test_sampleset")

    opt_value = 3.0
    pr = 0.7
    f = TimeToSolution()
    tts = f([sampleset], opt_value=opt_value, pr=pr)

    assert round(tts.data, 3) == 2.357

    tts = f([sampleset], pr, base="feasible")
    assert tts.data == 1.0

    tts = f([sampleset], pr, base="derived")
    assert round(tts.data, 3) == 2.357


def test_residual_energy(jm_sampleset: jm.SampleSet):
    from jijbench.functions.metrics import ResidualEnergy

    sampleset = jb.SampleSet(jm_sampleset, "test_sampleset")

    opt_value = 3.0
    f = ResidualEnergy()
    residual_energy = f([sampleset], opt_value=opt_value)

    assert residual_energy.data == 9.0


def test_evaluate_benchmark_results(sample_model):
    bench = jb.Benchmark({"num_reads": [1, 2]}, solver=sample_model)
    res = bench(autosave=False)

    opt_value = 3.0
    pr = 0.7
    evaluation = jb.Evaluation()
    res = evaluation([res], opt_value=opt_value, pr=pr)
